from __future__ import annotations

from requests import get

from .models import MapDataResponse, WarReportResponse, WarResponse

LIVE_API_URL = "https://war-service-live.foxholeservices.com/api"
LIVE2_API_URL = "https://war-service-live-2.foxholeservices.com/api"
LIVE3_API_URL = "https://war-service-live-3.foxholeservices.com/api"


class Api:
    def __init__(
        self, api_url: str | None, api_endpoint: str, timeout: int | None = 15
    ):
        self.api_base_url = LIVE_API_URL if api_url is None else api_url
        self.api_endpoint = api_endpoint
        self.timeout = timeout

    def _api_request(self, endpoint: str):
        url = f"{self.api_base_url}/{self.api_endpoint}/{endpoint}"
        response_json = get(url, timeout=self.timeout).json()
        return response_json


class WorldConquestApi(Api):
    def __init__(self, api_url: str | None, timeout: int | None = 15):
        self.api_endpoint = "worldconquest"
        super().__init__(
            api_url=api_url, api_endpoint=self.api_endpoint, timeout=timeout
        )

    def maps(self) -> list[str]:
        return self._api_request("maps")

    def maps_dynamic_public(self, map_name: str) -> MapDataResponse:
        endpoint = f"maps/{map_name}/dynamic/public"
        return self._api_request(endpoint)

    def maps_static(self, map_name: str) -> MapDataResponse:
        endpoint = f"maps/{map_name}/static"
        return self._api_request(endpoint)

    def war(self) -> WarResponse:
        return self._api_request("war")

    def war_report(self, map_name: str) -> WarReportResponse:
        endpoint = f"warReport/{map_name}"
        return self._api_request(endpoint)



from __future__ import annotations

from enum import Flag, IntEnum, StrEnum, UNIQUE, verify
from typing import TypedDict


@verify(UNIQUE)
class BaseFlags(Flag):
    VictoryBase = 0x01
    BuildSite = 0x04
    Scorched = 0x10  # v0.22
    TownClaimed = 0x20  # v0.26


@verify(UNIQUE)
class Faction(StrEnum):
    NONE = "NONE"
    COLONIALS = "COLONIALS"
    WARDENS = "WARDENS"


@verify(UNIQUE)
class MapIcon(IntEnum):
    ForwardBase = 8
    Hospital = 11
    VehicleFactory = 12
    Refinery = 17
    Shipyard = 18
    EngineeringCenter = 19
    SalvageField = 20
    ComponentField = 21
    SulfurField = 23
    WorldMapTent = 24
    TravelTent = 25
    TrainingArea = 26
    Keep = 27
    ObservationTower = 28
    Fort = 29
    TroopShip = 30
    SulfurMine = 32
    StorageFacility = 33
    Factory = 34
    GarrisonStation = 35
    RocketSite = 37
    SalvageMine = 38
    ConstructionYard = 39
    ComponentMine = 40
    RelicBase = 45
    BunkerBaseHusk = 48
    BunkerBaseHusk2 = 49
    BunkerBaseHusk3 = 50
    MassProductionFactory = 51
    Seaport = 52
    CoastalGun = 53
    SoulFactory = 54
    ClaimableBorderBase = 55
    TownBase = 56
    TownBase2 = 57
    TownBase3 = 58
    StormCannon = 59
    IntelCenter = 60
    CoalField = 61
    OilField = 62
    RocketTarget = 70
    RocketGroundZero = 71
    RocketSiteWithRocket = 72
    FacilityMineOilRig = 75


class MapItem(TypedDict):
    teamId: Faction
    iconType: MapIcon
    x: float
    y: float
    flags: int


class MapTextItem(TypedDict):
    text: str
    x: float
    y: float
    mapMarkerType: str


class MapDataResponse(TypedDict):
    regionId: int
    scorchedVictoryTowns: int
    mapItems: list[MapItem]
    mapItemsC: list[MapItem]
    mapItemsW: list[MapItem]
    mapTextItems: list[MapTextItem]
    lastUpdated: int
    version: int


class WarResponse(TypedDict):
    warId: str
    warNumber: int
    winner: Faction
    conquestStartTime: int
    conquestEndTime: int
    resistanceStartTime: int
    requiredVictoryTowns: int


class WarReportResponse(TypedDict):
    totalEnlistments: int
    colonialCasualties: int
    wardenCasualties: int
    dayOfWar: int



from .api import LIVE2_API_URL, LIVE3_API_URL, LIVE_API_URL, WorldConquestApi
from .models import (
    BaseFlags,
    Faction,
    MapDataResponse,
    MapIcon,
    MapItem,
    MapTextItem,
    WarReportResponse,
    WarResponse,
)

__all__ = [
    "BaseFlags",
    "Faction",
    "LIVE_API_URL",
    "LIVE2_API_URL",
    "LIVE3_API_URL",
    "MapDataResponse",
    "MapIcon",
    "MapItem",
    "MapTextItem",
    "WarResponse",
    "WarReportResponse",
    "WorldConquestApi",
]
